import os
import string
from pathlib import Path

CHARS_ALLOWED = string.ascii_lowercase + string.digits
SEPS = '_- /'
BRANCH = 'release'

CAPABILITY_DEFAULT = 'default'
RECONNECT_DELAY_SEC = 5
ON = 'ON'
OFF = 'OFF'

CONFIGS_PATH = Path(os.environ.get('HACO_CONFIGS_PATH','/haco/config'))
OPTIONS_PATH = Path(os.environ.get('HACO_OPTIONS_PATH','/haco/data/options.json'))
MQTT_HOST = os.environ.get('MQTT_HOST','localhost')
MQTT_PORT = int(os.environ.get('MQTT_PORT',1883))
MQTT_USERNAME = os.environ.get('MQTT_USERNAME')
MQTT_PASSWORD = os.environ.get('MQTT_PASSWORD',None)

if (CONFIGS_PATH / '.debug').exists():
    LOG_LEVEL = 'DEBUG'
else:
    LOG_LEVEL = os.environ.get('HACO_LOG_LEVEL', 'INFO')



from haco import Button, Tasmota

restart = Button('Restart Tasmota', icon='restart')


@restart.callback()
def ha(value) -> Tasmota[str, 'tasmota.cmd("restart 1")']:
    return value


from haco import Sensor

memory = Sensor('Memory Free', icon='memory', uom=Sensor.UOM.DATA_KILOBYTES, uom_type=Sensor.UOM_TYPE.DATA_SIZE)


@memory.callback(trigger='Tele#Heap')
def tasmota(value: int):
    return value


from haco import Button, Tasmota

restart = Button('Restart Tasmota', icon='restart')


@restart.callback()
def ha(value) -> Tasmota[str, 'tasmota.cmd("restart 1")']:
    return value


from haco import Select, Tasmota

LOG_LEVELS = ["None", "Error", "Info", "Debug", "Debug More"]

weblog = Select('WebLog Level', icon='console', options=LOG_LEVELS)


@weblog.callback()
def ha(value) -> Tasmota[str, 'tasmota.cmd("WebLog "+value)']:
    return LOG_LEVELS.index(value)


@weblog.callback(trigger='WebLog')
def tasmota(value: int):
    return LOG_LEVELS[value]



import random
from typing import Any

from haco import trigger, Update, Response
from haco.sensor import Sensor
from haco.tasmota import Tasmota
from haco.tools import get_latest


def get_memory_sensor():
    memory_sensor = Sensor('Memory Free', uom='kB', uom_type='DATA_SIZE', data_type=int)

    @memory_sensor.callback(trigger.Cron('*/30 * * * * *'))
    def tasmota(value: Tasmota[Any, 'tasmota.memory()']):
        return value.get('heap_free', 'unknown')

    return memory_sensor


def get_tasmota_updater():
    DEFAULT_OTA_URL = 'https://ota.tasmota.com/tasmota32/release/tasmota32.bin'

    once_daily = trigger.Cron(f"0 {random.randint(0, 59)} {random.randint(0, 23)} * * *")

    update = Update('Update Tasmota')

    @update.callback()
    def ha(value) -> Tasmota[str, 'tasmota.cmd("upgrade 1")']:
        return value

    @update.callback(trigger=once_daily)
    def latest_version_tasmota(value):
        tag = get_latest('arendst', 'Tasmota')

        if not tag:
            return Response(send=False)

        tag = tag.lstrip('v')

        return tag

    @update.callback(trigger=once_daily)
    def tasmota(value: Tasmota[Any, 'tasmota.cmd("status 2")'], otaurl: Tasmota[Any, 'tasmota.cmd("OtaUrl")']):

        if otaurl['OtaUrl'] != DEFAULT_OTA_URL:
            return Response(send=False)

        version = value.get('StatusFWR', {}).get('Version') or ''
        version = version.removesuffix('(tasmota32)')

        if not version:
            return Response(send=False)

        return version

    return update



from haco import Select, Tasmota

greeter = Select('Greeter', icon='hand-wave', options=['Hi', 'Hello Word', 'Bienvenue'])

set_greeting = """
def (value,data)

    import string
    var cmd=string.format('var16 %s',value)
    tasmota.cmd(cmd)
    
    print(string.format('haco says: %s!',value))

end
"""


@greeter.callback()
def ha(value) -> Tasmota[str, set_greeting]:
    return value


@greeter.callback(trigger='WebLog')
def tasmota(value: int):
    return value



from haco import tuya
from .button import Button
from .callback import Response
from .climate import Climate
from .fan import Fan
from .number import Number
from .pulldown import Select
from .sensor import Sensor
from .switch import Switch
from .tasmota import Tasmota
from .text import Text, Password
from .update import Update

__all__ = [
    "Button",
    "Select",
    "Sensor",
    "Tasmota",
    "Number",
    "Switch",
    "Text",
    "Password",
    "Update",
    "Climate",
    "Fan",
    "Response",
    "tuya"
]



class DeviceClassBinarySensor:
    """

    Sourced from:
    https://www.home-assistant.io/integrations/binary_sensor/#device-class

    """
    NONE: type(None) = None  # Generic on/off. This is the default and doesn’t need to be set.
    BATTERY: str = "battery"  # on means low, off means normal
    BATTERY_CHARGING: str = "battery_charging"  # on means charging, off means not charging
    CARBON_MONOXIDE: str = "carbon_monoxide"  # on means carbon monoxide detected, off no carbon monoxide (clear)
    COLD: str = "cold"  # on means cold, off means normal
    CONNECTIVITY: str = "connectivity"  # on means connected, off means disconnected
    DOOR: str = "door"  # on means open, off means closed
    GARAGE_DOOR: str = "garage_door"  # on means open, off means closed
    GAS: str = "gas"  # on means gas detected, off means no gas (clear)
    HEAT: str = "heat"  # on means hot, off means normal
    LIGHT: str = "light"  # on means light detected, off means no light
    LOCK: str = "lock"  # on means open (unlocked), off means closed (locked)
    MOISTURE: str = "moisture"  # on means moisture detected (wet), off means no moisture (dry)
    MOTION: str = "motion"  # on means motion detected, off means no motion (clear)
    MOVING: str = "moving"  # on means moving, off means not moving (stopped)
    OCCUPANCY: str = "occupancy"  # on means occupied (detected), off means not occupied (clear)
    OPENING: str = "opening"  # on means open, off means closed
    PLUG: str = "plug"  # on means device is plugged in, off means device is unplugged
    POWER: str = "power"  # on means power detected, off means no power
    PRESENCE: str = "presence"  # on means home, off means away
    PROBLEM: str = "problem"  # on means problem detected, off means no problem (OK)
    RUNNING: str = "running"  # on means running, off means not running
    SAFETY: str = "safety"  # on means unsafe, off means safe
    SMOKE: str = "smoke"  # on means smoke detected, off means no smoke (clear)
    SOUND: str = "sound"  # on means sound detected, off means no sound (clear)
    TAMPER: str = "tamper"  # on means tampering detected, off means no tampering (clear)
    UPDATE: str = "update"  # on means update available, off means up-to-date
    VIBRATION: str = "vibration"  # on means vibration detected, off means no vibration (clear)
    WINDOW: str = "window"  # on means open, off means closed



class Uom:
    """

    Sourced from: https://github.com/home-assistant/core/blob/03b1c7ad1deb4b9494df01b27ef8e1d7dfe179ba/homeassistant/const.py#L489C1-L489C1

    """
    POWER_WATT: str = "W"
    POWER_KILO_WATT: str = "kW"

    VOLT: str = "V"

    ENERGY_WATT_HOUR: str = "Wh"
    ENERGY_KILO_WATT_HOUR: str = "kWh"

    ELECTRICAL_CURRENT_AMPERE: str = "A"
    ELECTRICAL_VOLT_AMPERE: str = "VA"

    DEGREE: str = "°"

    CURRENCY_EURO: str = "€"
    CURRENCY_DOLLAR: str = "$"
    CURRENCY_CENT: str = "¢"

    TEMP_CELSIUS: str = "°C"
    TEMP_FAHRENHEIT: str = "°F"
    TEMP_KELVIN: str = "K"

    TIME_MICROSECONDS: str = "μs"
    TIME_MILLISECONDS: str = "ms"
    TIME_SECONDS: str = "s"
    TIME_MINUTES: str = "min"
    TIME_HOURS: str = "h"
    TIME_DAYS: str = "d"
    TIME_WEEKS: str = "w"
    TIME_MONTHS: str = "m"
    TIME_YEARS: str = "y"

    LENGTH_MILLIMETERS: str = "mm"
    LENGTH_CENTIMETERS: str = "cm"
    LENGTH_METERS: str = "m"
    LENGTH_KILOMETERS: str = "km"

    LENGTH_INCHES: str = "in"
    LENGTH_FEET: str = "ft"
    LENGTH_YARD: str = "yd"
    LENGTH_MILES: str = "mi"

    FREQUENCY_HERTZ: str = "Hz"
    FREQUENCY_GIGAHERTZ: str = "GHz"

    PRESSURE_PA: str = "Pa"
    PRESSURE_HPA: str = "hPa"
    PRESSURE_BAR: str = "bar"
    PRESSURE_MBAR: str = "mbar"
    PRESSURE_INHG: str = "inHg"
    PRESSURE_PSI: str = "psi"

    VOLUME_LITERS: str = "L"
    VOLUME_MILLILITERS: str = "mL"
    VOLUME_CUBIC_METERS: str = "m³"
    VOLUME_CUBIC_FEET: str = "ft³"

    VOLUME_GALLONS: str = "gal"
    VOLUME_FLUID_OUNCE: str = "fl. oz."

    VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR: str = "m³/h"
    VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE: str = "ft³/m"

    AREA_SQUARE_METERS: str = "m²"

    MASS_GRAMS: str = "g"
    MASS_KILOGRAMS: str = "kg"
    MASS_MILLIGRAMS: str = "mg"
    MASS_MICROGRAMS: str = "µg"

    MASS_OUNCES: str = "oz"
    MASS_POUNDS: str = "lb"

    CONDUCTIVITY: str = "µS/cm"

    LIGHT_LUX: str = "lx"

    UV_INDEX: str = "UV index"

    PERCENTAGE: str = "%"

    IRRADIATION_WATTS_PER_SQUARE_METER: str = "W/m²"

    PRECIPITATION_MILLIMETERS_PER_HOUR: str = "mm/h"

    CONCENTRATION_MICROGRAMS_PER_CUBIC_METER: str = "µg/m³"
    CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER: str = "mg/m³"
    CONCENTRATION_PARTS_PER_CUBIC_METER: str = "p/m³"
    CONCENTRATION_PARTS_PER_MILLION: str = "ppm"
    CONCENTRATION_PARTS_PER_BILLION: str = "ppb"

    SPEED_MILLIMETERS_PER_DAY: str = "mm/d"
    SPEED_INCHES_PER_DAY: str = "in/d"
    SPEED_METERS_PER_SECOND: str = "m/s"
    SPEED_INCHES_PER_HOUR: str = "in/h"
    SPEED_KILOMETERS_PER_HOUR: str = "km/h"
    SPEED_MILES_PER_HOUR: str = "mph"

    SIGNAL_STRENGTH_DECIBELS: str = "dB"
    SIGNAL_STRENGTH_DECIBELS_MILLIWATT: str = "dBm"

    DATA_BITS: str = "bit"
    DATA_KILOBITS: str = "kbit"
    DATA_MEGABITS: str = "Mbit"
    DATA_GIGABITS: str = "Gbit"
    DATA_BYTES: str = "B"
    DATA_KILOBYTES: str = "kB"
    DATA_MEGABYTES: str = "MB"
    DATA_GIGABYTES: str = "GB"
    DATA_TERABYTES: str = "TB"
    DATA_PETABYTES: str = "PB"
    DATA_EXABYTES: str = "EB"
    DATA_ZETTABYTES: str = "ZB"
    DATA_YOTTABYTES: str = "YB"
    DATA_KIBIBYTES: str = "KiB"
    DATA_MEBIBYTES: str = "MiB"
    DATA_GIBIBYTES: str = "GiB"
    DATA_TEBIBYTES: str = "TiB"
    DATA_PEBIBYTES: str = "PiB"
    DATA_EXBIBYTES: str = "EiB"
    DATA_ZEBIBYTES: str = "ZiB"
    DATA_YOBIBYTES: str = "YiB"
    DATA_RATE_BITS_PER_SECOND: str = "bit/s"
    DATA_RATE_KILOBITS_PER_SECOND: str = "kbit/s"
    DATA_RATE_MEGABITS_PER_SECOND: str = "Mbit/s"
    DATA_RATE_GIGABITS_PER_SECOND: str = "Gbit/s"
    DATA_RATE_BYTES_PER_SECOND: str = "B/s"
    DATA_RATE_KILOBYTES_PER_SECOND: str = "kB/s"
    DATA_RATE_MEGABYTES_PER_SECOND: str = "MB/s"
    DATA_RATE_GIGABYTES_PER_SECOND: str = "GB/s"
    DATA_RATE_KIBIBYTES_PER_SECOND: str = "KiB/s"
    DATA_RATE_MEBIBYTES_PER_SECOND: str = "MiB/s"
    DATA_RATE_GIBIBYTES_PER_SECOND: str = "GiB/s"



from haco.control import Control
from haco.schema import Schema, Capability, AnnounceTopic, Tasmota, HomeAssistant


class Update(Control):
    NAME = 'update'

    @classmethod
    def get_schema(cls):
        schema = Schema(
            capabilities=[
                Capability(
                    ha=HomeAssistant(announce_data=AnnounceTopic(key='{io_ha}_topic')),
                    tasmota=Tasmota(announce_data=AnnounceTopic(key='{io_ha}_topic'))
                ),
                Capability(
                    name='latest_version',
                    ha=None,
                    tasmota=Tasmota(announce_data=AnnounceTopic(key='{capability}_topic'))
                )
            ]
        )
        return schema

    #    def init(name, entity_id, icon, temperature_unit, temperature_range, humidity_range, modes, preset_modes, fan_modes, swing_modes, precision, callbacks)

    def __init__(self, name=None, icon=None, release_url=None, entity_picture=None):
        self.release_url = release_url
        self.entity_picture = entity_picture

        super().__init__(name, icon=icon)

    def get_config_ha_ex(self):
        data = {
            'entity_picture': self.entity_picture,
            'payload_install': 'INSTALL',
            'release_url': self.release_url
        }

        return data



import aiomqtt
import asyncio
import json
from pathlib import Path

from haco import constants, tools
from haco.constants import MQTT_PASSWORD, MQTT_HOST, MQTT_PORT, MQTT_USERNAME
from haco.device import Device
from haco.device_data import DeviceData
from haco.load_configs import load_devices
from haco.tools import log_received, logger


async def listen(client, devices):
    TOPIC_ANNOUNCE = f"haco/{constants.BRANCH}/+/announce"
    await tools.log_subscribe(client, TOPIC_ANNOUNCE)

    async with client.messages() as messages:

        async for message in messages:
            await log_received(message)
            if message.topic.matches(TOPIC_ANNOUNCE):

                if not message.payload:
                    msg = f'Topic {message.topic.value} got empty payload. It may have been deleted.'
                    tools.logger.info(msg)
                    continue

                data_announce = json.loads(message.payload)
                data_announce = DeviceData(**data_announce)

                for identifier in data_announce.identifiers:
                    device: Device = devices.get(identifier)
                    if device:
                        break

                if not device:
                    msg = f"""Found device with no configuration module assigned. Assign one in Add-On Configuration: "{data_announce.name}" Hostname: {data_announce.hostname} MAC:{data_announce.mac}."""
                    tools.logger.info(msg)
                    continue

                if device.data_announce and data_announce.config_id == device.data_announce.config_id:
                    msg = f'Device "{device.data_announce.name}" successfully configured. Hostname: {device.data_announce.hostname} MAC:{device.data_announce.mac}'
                    tools.logger.info(msg)
                    continue

                await device.bind(client, data_announce)
                devices[device.data_announce.hostname] = device


            else:

                if message.topic.matches('haco/release/+/config/#'):
                    logger.error(f'Control code got config message. Do not know why.')
                    continue

                _, _, device_name, _, control, capability_id, platform, io = Path(message.topic.value).parts
                device = devices[device_name]
                await device.get(control).process_message(client, capability_id, platform, message)


async def start_mqtt():
    devices = load_devices()
    devices = {device.identifier: device for device in devices}

    while True:
        client = aiomqtt.Client(hostname=MQTT_HOST, port=MQTT_PORT, username=MQTT_USERNAME,
                                password=MQTT_PASSWORD)
        try:
            async with client:
                await asyncio.gather(listen(client, devices))
        except aiomqtt.MqttError:
            msg = f"MQTT: Connection lost; Reconnecting in {constants.RECONNECT_DELAY_SEC} seconds ..."
            logger.info(msg)
            await asyncio.sleep(constants.RECONNECT_DELAY_SEC)



from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict


@dataclass
class DeviceData:
    wifi: Dict[str, any]
    device_name: str
    fulltopic: str
    hostname: str
    mac: str
    version: str
    topic: str
    config_id: str
    eth: Dict[str, any]
    timestamp: Dict[str, int]
    prefix: Dict[str, str]
    prefix_cmnd: str = field(init=False)
    prefix_stat: str = field(init=False)
    prefix_tele: str = field(init=False)

    name: str = field(init=False)

    def __post_init__(self):
        # Unpack the 'prefix' argument into individual fields
        self.prefix_cmnd = self.prefix['Prefix1']
        self.prefix_stat = self.prefix['Prefix2']
        self.prefix_tele = self.prefix['Prefix3']

        # Replace placeholders in the fulltopic with actual values
        self.fulltopic = self.fulltopic.replace('%topic%', self.topic)

        self.name = self.device_name

    @property
    def identifiers(self):
        return {self.wifi.get('mac'), self.eth.get('mac'), self.hostname, self.mac}

    @property
    def topic_tele(self):
        topic = Path(self.fulltopic.replace('%prefix%', self.prefix_tele))
        return topic

    @property
    def topic_lwt_prop(self):
        topic = self.topic_tele / 'LWT'
        return topic



from haco import constants
from haco.control import Control
from haco.schema import Schema, Capability, AnnounceTopic, Tasmota, ControlType, HomeAssistant
from haco.tools import get_range_pair, invert_bool


class Fan(Control):
    NAME = 'fan'

    @classmethod
    def get_schema(cls):
        schema = Schema(
            capabilities=[
                Capability(
                    ha=HomeAssistant(announce_data=AnnounceTopic(key='{io_ha}_topic')),
                    tasmota=Tasmota(announce_data=AnnounceTopic(key='{io_ha}_topic'))
                ),
                Capability(
                    name='preset_mode',
                    type=str
                ),
                Capability(
                    name='percentage',
                    type=int
                ),
                Capability(
                    name='oscillation',
                    type=ControlType
                )

            ]
        )
        return schema

    def __init__(self, name=None, preset_modes=None, icon=None, speed_range=None):
        self.temperature_unit = speed_range
        self.speed_min, self.speed_max = get_range_pair(speed_range)
        self.preset_modes = preset_modes

        super().__init__(name, icon=icon)

    def get_config_ha_ex(self):
        data = {
            'speed_range_min': self.speed_min,
            'speed_range_max': self.speed_max,
            'preset_modes': self.preset_modes,
            'payload_oscillation_on': constants.ON,
            'payload_oscillation_off': constants.OFF,

        }

        return data

    @property
    def control_type(self):
        return invert_bool



class DeviceClassSensor:
    """

    Sourced from:
    https://www.home-assistant.io/integrations/sensor/#device-class

    """
    NONE: type(None) = None  # Generic sensor. This is the default and doesn’t need to be set.
    APPARENT_POWER: str = "apparent_power"  # Apparent power in VA.
    AQI: str = "aqi"  # Air Quality Index (unitless).
    ATMOSPHERIC_PRESSURE: str = "atmospheric_pressure"  # Atmospheric pressure in cbar, bar, hPa, inHg, kPa, mbar, Pa, or psi.
    BATTERY: str = "battery"  # Percentage of battery that is left in %.
    CARBON_DIOXIDE: str = "carbon_dioxide"  # Carbon Dioxide in CO2 (Smoke) in ppm.
    CARBON_MONOXIDE: str = "carbon_monoxide"  # Carbon Monoxide in CO (Gas CNG/LPG) in ppm.
    CURRENT: str = "current"  # Current in A or mA.
    DATA_RATE: str = "data_rate"  # Data rate in bit/s, kbit/s, Mbit/s, Gbit/s, B/s, kB/s, MB/s, GB/s, KiB/s, MiB/s, or GiB/s.
    DATA_SIZE: str = "data_size"  # Data size in bit, kbit, Mbit, Gbit, B, kB, MB, GB, TB, PB, EB, ZB, YB, KiB, MiB, GiB, TiB, PiB, EiB, ZiB, or YiB.
    DATE: str = "date"  # Date string (ISO 8601).
    DISTANCE: str = "distance"  # Generic distance in km, m, cm, mm, mi, yd, or in.
    DURATION: str = "duration"  # Duration in d, h, min, or s.
    ENERGY: str = "energy"  # Energy in Wh, kWh, MWh, MJ, or GJ.
    ENERGY_STORAGE: str = "energy_storage"  # Stored energy in Wh, kWh, MWh, MJ, or GJ.
    ENUM: str = "enum"  # Has a limited set of (non-numeric) states.
    FREQUENCY: str = "frequency"  # Frequency in Hz, kHz, MHz, or GHz.
    GAS: str = "gas"  # Gas volume in m³, ft³, or CCF.
    HUMIDITY: str = "humidity"  # Percentage of humidity in the air in %.
    ILLUMINANCE: str = "illuminance"  # The current light level in lx.
    IRRADIANCE: str = "irradiance"  # Irradiance in W/m² or BTU/(h⋅ft²).
    MOISTURE: str = "moisture"  # Percentage of water in a substance in %.
    MONETARY: str = "monetary"  # The monetary value (ISO 4217).
    NITROGEN_DIOXIDE: str = "nitrogen_dioxide"  # Concentration of Nitrogen Dioxide in µg/m³.
    NITROGEN_MONOXIDE: str = "nitrogen_monoxide"  # Concentration of Nitrogen Monoxide in µg/m³.
    NITROUS_OXIDE: str = "nitrous_oxide"  # Concentration of Nitrous Oxide in µg/m³.
    OZONE: str = "ozone"  # Concentration of Ozone in µg/m³.
    PH: str = "ph"  # Potential hydrogen (pH) value of a water solution.
    PM1: str = "pm1"  # Concentration of particulate matter less than 1 micrometer in µg/m³.
    PM25: str = "pm25"  # Concentration of particulate matter less than 2.5 micrometers in µg/m³.
    PM10: str = "pm10"  # Concentration of particulate matter less than 10 micrometers in µg/m³.
    POWER_FACTOR: str = "power_factor"  # Power factor (unitless), unit may be None or %.
    POWER: str = "power"  # Power in W or kW.
    PRECIPITATION: str = "precipitation"  # Accumulated precipitation in cm, in, or mm.
    PRECIPITATION_INTENSITY: str = "precipitation_intensity"  # Precipitation intensity in in/d, in/h, mm/d, or mm/h.
    PRESSURE: str = "pressure"  # Pressure in Pa, kPa, hPa, bar, cbar, mbar, mmHg, inHg, or psi.
    REACTIVE_POWER: str = "reactive_power"  # Reactive power in var.
    SIGNAL_STRENGTH: str = "signal_strength"  # Signal strength in dB or dBm.
    SOUND_PRESSURE: str = "sound_pressure"  # Sound pressure in dB or dBA.
    SPEED: str = "speed"  # Generic speed in ft/s, in/d, in/h, km/h, kn, m/s, mph, or mm/d.
    SULPHUR_DIOXIDE: str = "sulphur_dioxide"  # Concentration of sulphur dioxide in µg/m³.
    TEMPERATURE: str = "temperature"  # Temperature in °C, °F, or K.
    TIMESTAMP: str = "timestamp"  # Datetime object or timestamp string (ISO 8601).
    VOLATILE_ORGANIC_COMPOUNDS: str = "volatile_organic_compounds"  # Concentration of volatile organic compounds in µg/m³.
    VOLATILE_ORGANIC_COMPOUNDS_PARTS: str = "volatile_organic_compounds_parts"  # Ratio of volatile organic compounds in ppm or ppb.
    VOLTAGE: str = "voltage"  # Voltage in V or mV.
    VOLUME: str = "volume"  # Generic volume in L, mL, gal, fl. oz., m³, ft³, or CCF.
    VOLUME_STORAGE: str = "volume_storage"  # Generic stored volume in L, mL, gal, fl. oz., m³, ft³, or CCF.
    WATER: str = "water"  # Water consumption in L, gal, m³, ft³, or CCF.
    WEIGHT: str = "weight"  # Generic mass in kg, g, mg, µg, oz, lb, or st.
    WIND_SPEED: str = "wind_speed"  # Wind speed in ft/s, km/h, kn, m/s, or mph.



from importlib import reload

import importlib.util
import json
from pathlib import Path

from haco import tools
from haco.constants import CONFIGS_PATH, OPTIONS_PATH
from haco.control import Control
from haco.device import Device


def load_json(file_path):
    with open(file_path, 'r') as json_file:
        data = json.loads(json_file.read())
    return data


def load_controls_from_directory():
    if not CONFIGS_PATH.exists():
        tools.logger.warning(f'Configs path does not exist. Will be created: {CONFIGS_PATH}')
        CONFIGS_PATH.mkdir()

    options = load_json(OPTIONS_PATH)

    controls = []
    for data in options['assignments']:

        name = data['config']
        identifier = data['identifier']

        args = {'identifier': identifier}

        py_file = CONFIGS_PATH / Path(name).with_suffix('.py')
        module_name = py_file.stem

        try:
            module = importlib.import_module(f'haco.configs.{module_name}')
            reload(module)
        except ImportError:
            spec = importlib.util.spec_from_file_location(module_name, str(py_file))
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)

        args['config_module'] = module

        for obj_name in dir(module):
            obj = getattr(module, obj_name)
            if obj_name == 'DEVICE_CONFIG':
                args.update(obj)
            if isinstance(obj, Control):
                args.setdefault('controls', [])
                args['controls'].append(obj)

        controls.append(args)

    return controls


def load_devices():
    devices_args = load_controls_from_directory()
    devices = [Device(**args) for args in devices_args]
    return devices
